/*
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#import <Cocoa/Cocoa.h>
#import "VMRegion.h"

#define NATIVE_ENDIAN_BYTE_ORDER 0
#define BIG_ENDIAN_BYTE_ORDER    1
#define LITTLE_ENDIAN_BYTE_ORDER 2

@interface HDTVirtualMemory : NSObject

+ (NSNumber *)readInt8AtProcess:(pid_t)process
				  memoryAddress:(mach_vm_address_t)address;

+ (BOOL)writeInt8AtProcess:(pid_t)process
             memoryAddress:(mach_vm_address_t)address
                     value:(SInt8)value;
                     
+ (NSNumber *)readUnsignedInt8AtProcess:(pid_t)process
						  memoryAddress:(mach_vm_address_t)address;

+ (BOOL)writeUnsignedInt8AtProcess:(pid_t)process
                     memoryAddress:(mach_vm_address_t)address
                             value:(UInt8)value;
                                        
+ (NSNumber *)readInt16AtProcess:(pid_t)process
                   memoryAddress:(mach_vm_address_t)address
					   byteOrder:(int)byteOrder;

+ (BOOL)writeInt16AtProcess:(pid_t)process
              memoryAddress:(mach_vm_address_t)address
                      value:(SInt16)value
				  byteOrder:(int)byteOrder;
                      
+ (NSNumber *)readUnsignedInt16AtProcess:(pid_t)process
                           memoryAddress:(mach_vm_address_t)address
							   byteOrder:(int)byteOrder;

+ (BOOL)writeUnsignedInt16AtProcess:(pid_t)process
                      memoryAddress:(mach_vm_address_t)address
                              value:(UInt16)value
						  byteOrder:(int)byteOrder;

+ (NSNumber *)readInt32AtProcess:(pid_t)process
				   memoryAddress:(mach_vm_address_t)address
				byteOrder:(int)byteOrder;

+ (BOOL)writeInt32AtProcess:(pid_t)process
              memoryAddress:(mach_vm_address_t)address
                      value:(SInt32)value
				  byteOrder:(int)byteOrder;
                      
+ (NSNumber *)readUnsignedInt32AtProcess:(pid_t)process
                           memoryAddress:(mach_vm_address_t)address
							   byteOrder:(int)byteOrder;

+ (BOOL)writeUnsignedInt32AtProcess:(pid_t)process
                      memoryAddress:(mach_vm_address_t)address
                              value:(UInt32)value
						  byteOrder:(int)byteOrder;

+ (NSNumber *)readInt64AtProcess:(pid_t)process
                   memoryAddress:(mach_vm_address_t)address
					   byteOrder:(int)byteOrder;

+ (BOOL)writeInt64AtProcess:(pid_t)process
              memoryAddress:(mach_vm_address_t)address
                      value:(SInt64)value
				  byteOrder:(int)byteOrder;
                      
+ (NSNumber *)readUnsignedInt64AtProcess:(pid_t)process
                           memoryAddress:(mach_vm_address_t)address
							   byteOrder:(int)byteOrder;

+ (BOOL)writeUnsignedInt64AtProcess:(pid_t)process
                      memoryAddress:(mach_vm_address_t)address
                              value:(UInt64)value
						  byteOrder:(int)byteOrder;

+ (NSNumber *)readFloatAtProcess:(pid_t)process
                   memoryAddress:(mach_vm_address_t)address
					   byteOrder:(int)byteOrder;
						 
+ (BOOL)writeFloatAtProcess:(pid_t)process
              memoryAddress:(mach_vm_address_t)address
                      value:(float)value
				  byteOrder:(int)byteOrder;

+ (NSNumber *)readDoubleAtProcess:(pid_t)process
                    memoryAddress:(mach_vm_address_t)address
						byteOrder:(int)byteOrder;

+ (BOOL)writeDoubleAtProcess:(pid_t)process
               memoryAddress:(mach_vm_address_t)address
                       value:(double)value
				   byteOrder:(int)byteOrder;

+ (NSString *)readUTF8StringAtProcess:(pid_t)process
						memoryAddress:(mach_vm_address_t)address;
  
+ (BOOL)writeUTF8StringAtProcess:(pid_t)process
				   memoryAddress:(mach_vm_address_t)address
						   value:(NSString *)value;

+ (NSString *)readUTF16StringAtProcess:(pid_t)process
                        memoryAddress:(mach_vm_address_t)address
							 byteOrder:(int)byteOrder;

+ (BOOL)writeUTF16StringAtProcess:(pid_t)process
                   memoryAddress:(mach_vm_address_t)address
                           value:(NSString *)value
						byteOrder:(int)byteOrder;
@end
