#
#  HaloDemoMemoryTemplateAppDelegate.py
#  HaloDemoMemoryTemplate
#
#  Created by nil on 12/14/08.
#  Copyright 2008. All rights reserved.
#

from Foundation import *
from AppKit import *
from objc import *
import HaloDemoTrainer
from VirtualMemory import *
import threading

objc.informal_protocol("NSMenuValidation", [objc.selector(None, selector='validateMenuItem_', signature='b@:@')])      
class HaloDemoMemoryTemplateAppDelegate(NSObject):
  executeButton = objc.IBOutlet()
  descriptionTextField = objc.IBOutlet()
  aboutMenuItem = objc.IBOutlet()
  hideMenuItem = objc.IBOutlet()
  quitMenuItem = objc.IBOutlet()
  helpMenuItem = objc.IBOutlet()
  mainWindow = objc.IBOutlet()
  debugTextView = objc.IBOutlet()
  reloadScriptMenuItem = objc.IBOutlet()
  
  def setTitles(self):
    self.mainWindow.setTitle_(HaloDemoTrainer.WINDOW_TITLE)
    self.descriptionTextField.setStringValue_(NSString.stringWithFormat_("Description:\n\n %s", HaloDemoTrainer.DESCRIPTION))
  
  def awakeFromNib(self):
    NSApp.activateIgnoringOtherApps_(YES)
    bundleName = NSBundle.mainBundle().infoDictionary().objectForKey_("CFBundleName")
    self.aboutMenuItem.setTitle_("About " + bundleName)
    self.hideMenuItem.setTitle_("Hide " + bundleName)
    self.quitMenuItem.setTitle_("Quit " + bundleName)
    self.helpMenuItem.setTitle_(bundleName + " Help")
    self.setTitles()
    
  def applicationShouldTerminateAfterLastWindowClosed_(self, theApplication):
    return YES
      
  def applicationWillTerminate_(self, sender):
    if self.timer:
      HaloDemoTrainer.finish(self.timeElapsed, True)
  
  def applicationDidFinishLaunching_(self, sender):
    loadProcess()
    NSNotificationCenter.defaultCenter().addObserver_selector_name_object_(self, objc.selector(self.writeToDebugTextView, signature='v@:@'), "HDTDebugWrite", None)
    
    self.timer = None
    
    NSWorkspace.sharedWorkspace().notificationCenter().addObserver_selector_name_object_(self, objc.selector(self.applicationDidLaunch, signature='v@:@'), NSWorkspaceDidLaunchApplicationNotification, None)
    NSWorkspace.sharedWorkspace().notificationCenter().addObserver_selector_name_object_(self, objc.selector(self.applicationDidTerminate, signature='v@:@'), NSWorkspaceDidTerminateApplicationNotification, None)
    for applicationDictionary in NSWorkspace.sharedWorkspace().launchedApplications():
      self.findHaloDemo(applicationDictionary, True)
    
  def applicationDidLaunch(self, notification):
    self.findHaloDemo(notification.userInfo(), True)
    
  def applicationDidTerminate(self, notification):
    self.findHaloDemo(notification.userInfo(), False)
    
  def writeToDebugTextView(self, notification):
    self.debugTextView.textStorage().mutableString().appendString_(notification.userInfo().objectForKey_("HDTMessage"))
    self.scrollDebugTextView()
    
  def scrollDebugTextView(self):
    #Scroll to end of text view. Maybe there is a better way to do this...
    storageLength = self.debugTextView.textStorage().mutableString().length()
    self.debugTextView.scrollRangeToVisible_(NSMakeRange(storageLength, storageLength))
    
  @objc.IBAction
  def clearDebugTextView_(self, sender):
    self.debugTextView.textStorage().mutableString().setString_("")
    
  @objc.IBAction
  def reloadScript_(self, sender):
    reload(HaloDemoTrainer)
    self.setTitles()
    
  @objc.IBAction
  def execute_(self, sender):
    if self.executeButton.title() == "Start":
      reload(HaloDemoTrainer)
      self.setTitles()
      self.executeButton.setTitle_("Stop")
      self.timeElapsed = 0.0
      self.timer = NSTimer.scheduledTimerWithTimeInterval_target_selector_userInfo_repeats_(float(HaloDemoTrainer.EXECUTION_TIME_INTERVAL), self, objc.selector(self.callExecute , signature = 'v@:'), None, True)
      #execute first time
      self.callExecute()
    else:
      self.timer.invalidate()
      self.timer = None
      self.finishExecution(False)
      self.executeButton.setTitle_("Start")
      
  def finishExecution(self, forcedShutdown):
    try:
      HaloDemoTrainer.finish(self.timeElapsed, forcedShutdown)
    except Exception, exception:
      self.debugTextView.textStorage().mutableString().appendString_("Exception in finish(): " + str(exception) + "\n")
      self.scrollDebugTextView()
      
  def callExecute(self):
    try:
      HaloDemoTrainer.execute(self.timeElapsed)
      self.timeElapsed += HaloDemoTrainer.EXECUTION_TIME_INTERVAL
    except Exception, exception:
      self.timer.invalidate()
      self.timer = None;
      self.executeButton.setTitle_("Start")
      self.debugTextView.textStorage().mutableString().appendString_("Exception in execute(): " + str(exception) + "\n")
      self.scrollDebugTextView()
      self.finishExecution(True)
      
  def findHaloDemo(self, dictionary, aliveState):
    if dictionary.objectForKey_("NSApplicationName") == "Halo Demo":
      self.executeButton.setEnabled_(aliveState)
      
      if self.executeButton.title() == "Stop":
        #Halo Demo quit
        self.timer.invalidate()
        self.timer = None
        self.executeButton.setTitle_("Start")
  
  def validateMenuItem_(self, menuItem):
    if menuItem.title() == "Reload Script":
      if self.timer:
        return NO
        
    return YES